@@ -39,35 +39,49 @@
 
 #define TEXT_SNIFF_SIZE 4096
 
-gint
+FILE *
 tracker_file_open (const gchar *uri,
-		   gboolean	lreadahead)
+		   const gchar *how,
+		   gboolean	sequential)
 {
-	gint fd;
+ 	FILE     *file;
+	gboolean  readonly;
+	int       flags;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (how != NULL, NULL);
+
+	file = fopen (uri, how);
+	if (!file) {
+		return NULL;
+	}
+
+ 	/* Are we opening for readonly? */
+	readonly = !strstr (uri, "r+") && strchr (uri, 'r');
+
+	if (readonly) {
+		int fd;
+
+		fd = fileno (file);
+		
 #if defined(__linux__)
-	fd = open (uri, O_RDONLY | O_NOATIME);
-
-	if (fd == -1) {
-		fd = open (uri, O_RDONLY);
-	}
-#else
-	fd = open (uri, O_RDONLY);
+		/* Make sure we set the NOATIME flag if we have permissions to */
+		if ((flags = fcntl (fd, F_GETFL, 0)) != -1) {
+			fcntl (fd, F_SETFL, flags | O_NOATIME);
+		}
 #endif
-
-	if (fd == -1) {
-		return -1;
 	}
+	
+	return file;
+}
 
-#ifdef HAVE_POSIX_FADVISE
-	if (lreadahead) {
-		posix_fadvise (fd, 0, 0, POSIX_FADV_SEQUENTIAL);
-	} else {
-		posix_fadvise (fd, 0, 0, POSIX_FADV_RANDOM);
-	}
-#endif
+void
+tracker_file_close (FILE     *file,
+		    gboolean  need_again_soon) 
+{
+	g_return_if_fail (file != NULL);
 
-	return fd;
+	fclose (file);
 }
 
 gboolean
