@@ -48,18 +48,20 @@ static IptcTagType iptctags[] = {
 	/*	{ 2, IPTC_TAG_CONTENT_LOC_NAME, "Image:Location", NULL }, */
 	{ 2, IPTC_TAG_SUBLOCATION, "Image:Location", FALSE, NULL },
         { 2, IPTC_TAG_DATE_CREATED, "Image:Date", FALSE, NULL },
-        { 2, IPTC_TAG_ORIGINATING_PROGRAM, "Image:Software", FALSE, NULL },
         { 2, IPTC_TAG_BYLINE, "Image:Creator", FALSE, NULL },
         { 2, IPTC_TAG_CITY, "Image:City", FALSE, NULL },
         { 2, IPTC_TAG_COUNTRY_NAME, "Image:Country", FALSE, NULL },
         { 2, IPTC_TAG_CREDIT, "Image:Creator", FALSE, NULL },
         { 2, IPTC_TAG_COPYRIGHT_NOTICE, "File:Copyright", FALSE, NULL },
         { 2, IPTC_TAG_IMAGE_ORIENTATION, "Image:Orientation", FALSE, fix_iptc_orientation },
+#ifdef ENABLE_DETAILED_METADATA
+        { 2, IPTC_TAG_ORIGINATING_PROGRAM, "Image:Software", FALSE, NULL },
+#endif /* ENABLE_DETAILED_METADATA */
 	{ -1, -1, NULL, FALSE, NULL }
 };
 
 static void
-metadata_append (GHashTable *metadata, gchar *key, gchar *value, gboolean append)
+metadata_append (GHashTable *metadata, gchar *key, gchar *value, gboolean append, gboolean set_additional)
 {
 	gchar   *new_value;
 	gchar   *orig;
@@ -67,47 +69,55 @@ metadata_append (GHashTable *metadata, gchar *key, gchar *value, gboolean append
 	gboolean found = FALSE;
 	guint    i;
 
-	if (append && (orig = g_hash_table_lookup (metadata, key))) {
-		gchar *escaped;
-		
-		escaped = tracker_escape_metadata (value);
+	/* Iptc is always low priority, don't overwrite */
+	if ( (orig = g_hash_table_lookup (metadata, key)) != NULL) {
+		if (append) {
+			gchar *escaped;
 
-		list = g_strsplit (orig, "|", -1);			
-		for (i=0; list[i]; i++) {
-			if (strcmp (list[i], escaped) == 0) {
-				found = TRUE;
-				break;
+			escaped = tracker_escape_metadata (value);
+
+			list = g_strsplit (orig, "|", -1);
+			for (i=0; list[i]; i++) {
+				/* Check 32 first characters (truncated) */
+				if (strncmp (list[i], escaped, 32) == 0) {
+					found = TRUE;
+					break;
+				}
 			}
-		}			
-		g_strfreev(list);
+			g_strfreev(list);
 
-		if (!found) {
-			new_value = g_strconcat (orig, "|", escaped, NULL);
-			g_hash_table_insert (metadata, g_strdup (key), new_value);
+			if (!found) {
+				new_value = g_strconcat (orig, "|", escaped, NULL);
+				g_hash_table_insert (metadata, g_strdup (key), new_value);
+			}
+
+			g_free (escaped);
 		}
-
-		g_free (escaped);		
 	} else {
 		new_value = tracker_escape_metadata (value);
 		g_hash_table_insert (metadata, g_strdup (key), new_value);
 
 		/* FIXME Postprocessing is evil and should be elsewhere */
-		if (strcmp (key, "Image:Keywords") == 0) {
+		if (set_additional && strcmp (key, "Image:Keywords") == 0) {
 			g_hash_table_insert (metadata,
 					     g_strdup ("Image:HasKeywords"),
 					     tracker_escape_metadata ("1"));			
 		}		
 	}
 
-	/* Adding certain fields also to keywords FIXME Postprocessing is evil */
+	/* Giving certain properties HasKeywords FIXME Postprocessing is evil */
+	if ((strcmp (key, "Image:Title") == 0) ||
+	    (strcmp (key, "Image:Description") == 0) ) {
+		g_hash_table_insert (metadata,
+				     g_strdup ("Image:HasKeywords"),
+				     tracker_escape_metadata ("1"));
+	}
+	/* Adding certain fields also to keywords */
 	if ((strcmp (key, "Image:Location") == 0) ||
 	    (strcmp (key, "Image:Sublocation") == 0) ||
 	    (strcmp (key, "Image:Country") == 0) ||
 	    (strcmp (key, "Image:City") == 0) ) {
-		metadata_append (metadata, "Image:Keywords", value, TRUE);
-		g_hash_table_insert (metadata,
-				     g_strdup ("Image:HasKeywords"),
-				     tracker_escape_metadata ("1"));
+		metadata_append (metadata, "Image:Keywords", value, TRUE, FALSE);
 	}
 }
 
@@ -151,9 +161,9 @@ tracker_read_iptc (const unsigned char *buffer,
 			iptc_dataset_get_as_str (dataset, buffer, 1024);
 			
 			if (p->post) {
-				metadata_append (metadata,p->name,(*p->post) (buffer), p->multi);
+				metadata_append (metadata,p->name,(*p->post) (buffer), p->multi, TRUE);
 			} else {
-				metadata_append (metadata, p->name, buffer, p->multi);
+				metadata_append (metadata, p->name, buffer, p->multi, TRUE);
 			}
 		}
 	}
