@@ -33,6 +33,10 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <time.h>
+#if defined(__linux__)
+#include <linux/sched.h>
+#endif
+#include <sched.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -55,8 +59,9 @@
 
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-turtle.h>
+#include <libtracker-data/tracker-data-backup.h>
 
-#include <plugins/evolution/tracker-evolution.h>
+#include <tracker-push.h>
 
 #include "tracker-crawler.h"
 #include "tracker-dbus.h"
@@ -65,9 +70,9 @@
 #include "tracker-monitor.h"
 #include "tracker-processor.h"
 #include "tracker-status.h"
-#include "tracker-xesam-manager.h"
-#include "tracker-cleanup.h"
+#include "tracker-volume-cleanup.h"
 #include "tracker-backup.h"
+#include "tracker-daemon.h"
 
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -76,8 +81,7 @@
 #endif
 
 #define ABOUT								  \
-	"Tracker " PACKAGE_VERSION "\n"					  \
-	"Copyright (c) 2005-2008 Jamie McCracken (jamiemcc@gnome.org)\n"
+	"Tracker " PACKAGE_VERSION "\n"
 
 #define LICENSE								  \
 	"This program is free software and comes without any warranty.\n" \
@@ -90,6 +94,17 @@
 #define THROTTLE_DEFAULT	    0
 #define THROTTLE_DEFAULT_ON_BATTERY 5
 
+#ifdef HAVE_HAL 
+
+typedef struct {
+	gchar    *udi;
+	gchar    *mount_point;
+	gboolean  no_crawling;
+	gboolean  was_added;
+} MountPointUpdate;
+
+#endif /* HAVE_HAL */
+
 typedef enum {
 	TRACKER_RUNNING_NON_ALLOWED,
 	TRACKER_RUNNING_READONLY,
@@ -105,15 +120,22 @@ typedef struct {
 	gchar		 *sys_tmp_dir;
 	gchar            *ttl_backup_file;
 	
+	gboolean          public_interfaces_up;
+	gboolean          mount_points_up;
+
 	gboolean	  reindex_on_shutdown;
+	gboolean          shutdown;
+
+	gint              mount_points_to_set;
 
 	TrackerProcessor *processor;
 } TrackerMainPrivate;
 
 /* Private */
-static GStaticPrivate	private_key = G_STATIC_PRIVATE_INIT;
+static GStaticPrivate	     private_key = G_STATIC_PRIVATE_INIT;
 
 /* Private command line parameters */
+static gboolean		     version;
 static gint		     verbosity = -1;
 static gint		     initial_sleep = -1;
 static gboolean		     low_memory;
@@ -126,7 +148,12 @@ static gboolean		     force_reindex;
 static gboolean		     disable_indexing;
 static gchar	            *language_code;
 
-static GOptionEntry	entries_daemon[] = {
+static GOptionEntry	     entries[] = {
+	/* Daemon options */
+	{ "version", 'V', 0,
+	  G_OPTION_ARG_NONE, &version,
+	  N_("Displays version information"),
+	  NULL },
 	{ "verbosity", 'v', 0,
 	  G_OPTION_ARG_INT, &verbosity,
 	  N_("Logging, 0 = errors only, "
@@ -156,10 +183,8 @@ static GOptionEntry	entries_daemon[] = {
 	  G_OPTION_ARG_STRING_ARRAY, &disable_modules,
 	  N_("Disable modules from being processed (you can do -d <module> -d <module>)"),
 	  NULL },
-	{ NULL }
-};
 
-static GOptionEntry   entries_indexer[] = {
+	/* Indexer options */
 	{ "force-reindex", 'r', 0,
 	  G_OPTION_ARG_NONE, &force_reindex,
 	  N_("Force a re-index of all content"),
@@ -193,7 +218,9 @@ private_free (gpointer data)
 	g_free (private->ttl_backup_file);
 	g_free (private->log_filename);
 
-	g_main_loop_unref (private->main_loop);
+	if (private->main_loop) {
+		g_main_loop_unref (private->main_loop);
+	}
 
 	g_free (private);
 }
@@ -295,74 +322,11 @@ check_runtime_level (TrackerConfig *config,
 #endif /* HAVE_HAL */
 	}
 
+	close (fd);
+
 	return runlevel;
 }
 
-#ifdef HAVE_HAL
-
-static void
-mount_point_set_cb (DBusGProxy *proxy, 
-		    GError     *error, 
-		    gpointer    user_data)
-{
-	if (error) {
-		g_critical ("Couldn't set mount point state, %s", 
-			    error->message);
-		g_error_free (error);
-		g_free (user_data);
-		return;
-	}
-
-	g_message ("Indexer now knows about UDI state:");
-	g_message ("  %s", (gchar*) user_data);
-
-	g_free (user_data);
-}
-
-static void
-mount_point_added_cb (TrackerHal  *hal,
-		      const gchar *udi,
-		      const gchar *mount_point,
-		      gpointer	   user_data)
-{
-	TrackerMainPrivate *private;
-	
-	private = g_static_private_get (&private_key);
-
-	g_message ("Indexer is being notified about added UDI:");
-	g_message ("  %s", udi);
-
-	org_freedesktop_Tracker_Indexer_volume_update_state_async (tracker_dbus_indexer_get_proxy (), 
-								   udi,
-								   mount_point,
-								   TRUE,
-								   mount_point_set_cb,
-								   g_strdup (udi));
-}
-
-static void
-mount_point_removed_cb (TrackerHal  *hal,
-			const gchar *udi,
-			const gchar *mount_point,
-			gpointer     user_data)
-{
-	TrackerMainPrivate *private;
-	
-	private = g_static_private_get (&private_key);
-
-	g_message ("Indexer is being notified about removed UDI:");
-	g_message ("  %s", udi);
-
-	org_freedesktop_Tracker_Indexer_volume_update_state_async (tracker_dbus_indexer_get_proxy (), 
-								   udi,
-								   mount_point,
-								   FALSE,
-								   mount_point_set_cb,
-								   g_strdup (udi));
-}
-
-#endif /* HAVE_HAL */
-
 static void
 log_option_list (GSList      *list,
 		 const gchar *str)
@@ -396,7 +360,7 @@ sanity_check_option_values (TrackerConfig *config)
 	g_message ("Daemon options:");
 	g_message ("  Throttle level  .......................  %d",
 		   tracker_config_get_throttle (config));
-	g_message ("  Indexing enabled	.....................  %s",
+	g_message ("  Indexing enabled  .....................  %s",
 		   tracker_config_get_enable_indexing (config) ? "yes" : "no");
 	g_message ("  Monitoring enabled  ...................  %s",
 		   tracker_config_get_enable_watches (config) ? "yes" : "no");
@@ -423,25 +387,16 @@ shutdown_timeout_cb (gpointer user_data)
 }
 
 static void
-signal_handler (gint signo)
+signal_handler (int signo)
 {
 	static gboolean in_loop = FALSE;
 
 	/* Die if we get re-entrant signals handler calls */
 	if (in_loop) {
-		exit (EXIT_FAILURE);
+		_exit (EXIT_FAILURE);
 	}
 
 	switch (signo) {
-	case SIGSEGV:
-		/* We are screwed if we get this so exit immediately! */
-		exit (EXIT_FAILURE);
-
-	case SIGBUS:
-	case SIGILL:
-	case SIGFPE:
-	case SIGPIPE:
-	case SIGABRT:
 	case SIGTERM:
 	case SIGINT:
 		in_loop = TRUE;
@@ -449,9 +404,10 @@ signal_handler (gint signo)
 
 	default:
 		if (g_strsignal (signo)) {
-			g_message ("Received signal:%d->'%s'",
-				   signo,
-				   g_strsignal (signo));
+			g_print ("\n");
+			g_print ("Received signal:%d->'%s'",
+				 signo,
+				 g_strsignal (signo));
 		}
 		break;
 	}
@@ -461,8 +417,8 @@ static void
 initialize_signal_handler (void)
 {
 #ifndef G_OS_WIN32
-	struct sigaction   act;
-	sigset_t	   empty_mask;
+	struct sigaction act;
+	sigset_t	 empty_mask;
 
 	sigemptyset (&empty_mask);
 	act.sa_handler = signal_handler;
@@ -470,17 +426,23 @@ initialize_signal_handler (void)
 	act.sa_flags   = 0;
 
 	sigaction (SIGTERM, &act, NULL);
-	sigaction (SIGILL,  &act, NULL);
-	sigaction (SIGBUS,  &act, NULL);
-	sigaction (SIGFPE,  &act, NULL);
-	sigaction (SIGHUP,  &act, NULL);
-	sigaction (SIGSEGV, &act, NULL);
-	sigaction (SIGABRT, &act, NULL);
-	sigaction (SIGUSR1, &act, NULL);
 	sigaction (SIGINT,  &act, NULL);
+	sigaction (SIGHUP,  &act, NULL);
 #endif /* G_OS_WIN32 */
 }
 
+static void
+initialize_priority (void)
+{
+	/* Set disk IO priority and scheduling */
+	tracker_ioprio_init ();
+
+	/* NOTE: We only set the nice() value when crawling, for all
+	 * other times we don't have a nice() value. Check the
+	 * tracker-status code to see where this is done.
+	 */
+}
+
 static void
 initialize_locations (void)
 {
@@ -552,9 +514,6 @@ initialize_directories (void)
 	filename = g_build_filename (private->sys_tmp_dir, "Attachments", NULL);
 	g_mkdir_with_parents (filename, 00700);
 	g_free (filename);
-
-	/* Remove existing log files */
-	tracker_file_unlink (private->log_filename);
 }
 
 static gboolean
@@ -564,25 +523,7 @@ initialize_databases (void)
 	 * Create SQLite databases
 	 */
 	if (!tracker_status_get_is_readonly () && force_reindex) {
-		TrackerDBInterface *iface;
-
 		tracker_status_set_is_first_time_index (TRUE);
-
-		/* Reset stats for embedded services if they are being reindexed */
-
-		/* Here it doesn't matter which one we ask, as long as it has common.db
-		 * attached. The service ones are cached connections, so we can use
-		 * those instead of asking for an individual-file connection (like what
-		 * the original code had) */
-
-		/* iface = tracker_db_manager_get_db_interfaceX (TRACKER_DB_COMMON); */
-
-		iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
-
-		g_message ("*** DELETING STATS *** ");
-		tracker_data_manager_exec_no_reply (iface,
-					  "update ServiceTypes set TypeCount = 0 where Embedded = 1");
-
 	}
 
 	/* Check db integrity if not previously shut down cleanly */
@@ -596,12 +537,6 @@ initialize_databases (void)
 	return TRUE;
 }
 
-
-static void
-shutdown_indexer (void)
-{
-}
-
 static void
 shutdown_databases (void)
 {
@@ -611,7 +546,7 @@ shutdown_databases (void)
 
 	/* If we are reindexing, save the user metadata  */
 	if (private->reindex_on_shutdown) {
-		tracker_backup_save (private->ttl_backup_file);
+		tracker_data_backup_save (private->ttl_backup_file, NULL);
 	}
 	/* Reset integrity status as threads have closed cleanly */
 	tracker_data_manager_set_db_option_int ("IntegrityCheck", 0);
@@ -633,13 +568,13 @@ shutdown_directories (void)
 	/* If we are reindexing, just remove the databases */
 	if (private->reindex_on_shutdown) {
 		tracker_db_manager_remove_all ();
+		tracker_db_index_manager_remove_all ();
 	}
 }
 
 static const gchar *
 get_ttl_backup_filename (void) 
 {
-
 	TrackerMainPrivate *private;
 
 	private = g_static_private_get (&private_key);
@@ -680,7 +615,7 @@ backup_user_metadata (TrackerConfig *config, TrackerLanguage *language)
 	tracker_data_manager_init (config, language, file_index, email_index);
 	
 	/* Actual save of the metadata */
-	tracker_backup_save (get_ttl_backup_filename ());
+	tracker_data_backup_save (get_ttl_backup_filename (), NULL);
 	
 	/* Shutdown the DB stack */
 	tracker_data_manager_shutdown ();
@@ -695,25 +630,33 @@ backup_user_metadata (TrackerConfig *config, TrackerLanguage *language)
  * Saving the last backup file to help with debugging.
  */
 static void
-crawling_finished_cb (TrackerProcessor *processor, gpointer user_data)
+crawling_finished_cb (TrackerProcessor *processor, 
+		      gpointer          user_data)
 {
-	gulong *callback_id = user_data;
-	GError *error;
+	GError *error = NULL;
 	static gint counter = 0;
 	
-	counter += 1;
-
-	if (counter >= 2) {
+	if (++counter >= 2) {
 		gchar *rebackup;
 
 		g_debug ("Uninstalling initial crawling callback");
-		g_signal_handler_disconnect (processor, *callback_id);
+
+		g_signal_handlers_disconnect_by_func (processor, 
+						      crawling_finished_cb, 
+						      user_data);
 
 		if (g_file_test (get_ttl_backup_filename (), G_FILE_TEST_EXISTS)) {
-			org_freedesktop_Tracker_Indexer_restore_backup (tracker_dbus_indexer_get_proxy (), 
+			org_freedesktop_Tracker_Indexer_restore_backup (tracker_dbus_indexer_get_proxy (),
 									get_ttl_backup_filename (),
 									&error);
-		
+
+			if (error) {
+				g_message ("Could not restore backup, %s",
+					   error->message);
+				g_error_free (error);
+				return;
+			}
+
 			rebackup = g_strdup_printf ("%s.old",
 						    get_ttl_backup_filename ());
 			g_rename (get_ttl_backup_filename (), rebackup);
@@ -725,131 +668,269 @@ crawling_finished_cb (TrackerProcessor *processor, gpointer user_data)
 	}
 }
 
-static void
-backup_restore_on_crawling_finished (TrackerProcessor *processor)
-{
-	static gulong restore_cb_id = 0;
+#ifdef HAVE_HAL
 
-	g_debug ("Setting callback for crawling finish detection");
-	restore_cb_id = g_signal_connect (processor, "finished", 
-					  G_CALLBACK (crawling_finished_cb), 
-					  &restore_cb_id);
+static MountPointUpdate *
+mount_point_update_new (const gchar *udi,
+			const gchar *mount_point,
+			gboolean     no_crawling,
+			gboolean     was_added)
+{
+	MountPointUpdate *mpu;
+
+	mpu = g_slice_new0 (MountPointUpdate);
+
+	mpu->udi = g_strdup (udi);
+	mpu->mount_point = g_strdup (mount_point);
+
+	mpu->no_crawling = no_crawling;
+	mpu->was_added = was_added;
+
+	return mpu;
 }
 
-#ifdef HAVE_HAL
+static void
+mount_point_update_free (MountPointUpdate *mpu)
+{
+	if (!mpu) {
+		return;
+	}
+
+	g_free (mpu->mount_point);
+	g_free (mpu->udi);
+
+	g_slice_free (MountPointUpdate, mpu);
+}
+
+static void
+mount_point_set_cb (DBusGProxy *proxy, 
+		    GError     *error, 
+		    gpointer    user_data)
+{
+	TrackerMainPrivate *private;
+	MountPointUpdate   *mpu;
+
+	mpu = user_data;
+
+	if (error) {
+		g_critical ("Indexer couldn't set volume state for:'%s' in database, %s", 
+			    mpu->udi,
+			    error ? error->message : "no error given");
+
+		g_error_free (error);
+
+		tracker_shutdown ();
+	} else {
+		g_message ("Indexer has now set the state for the volume with UDI:");
+		g_message ("  %s", mpu->udi);
+	}
+		
+	/* See if we have any more callbacks here, if we don't we can
+	 * signal the stats to update.
+	 */
+	private = g_static_private_get (&private_key);
+
+	private->mount_points_to_set--;
+
+	if (private->mount_points_to_set < 1) {
+		/* This is a special case, because we don't get the
+		 * "Finished" signal from the indexer when we set something
+		 * in the volumes table, we have to signal all clients from
+		 * here that the statistics may have changed.
+		 */
+		if (private->public_interfaces_up) {
+			tracker_daemon_signal_statistics ();
+		}
+
+		/* Unpause the indexer */
+		tracker_status_set_is_paused_for_dbus (FALSE);
+			
+		/* Don't try to quit the main loop twice if we have
+		 * had an error above.
+		 */
+		if (!private->mount_points_up && 
+		    !private->shutdown) {
+			private->mount_points_up = TRUE;
+
+			/* We need to stop the main loop, we started
+			 * it so we could wait for mount points to be
+			 * set up successfully in main().
+			 */
+			g_main_loop_quit (private->main_loop);
+		}
+	} else if (private->mount_points_up) {
+		g_message ("Statistics not being signalled, %d %s remaining", 
+			   private->mount_points_to_set,
+			   private->mount_points_to_set == 1 ? "volume" : "volumes");
+	}
+
+	/* Make sure we crawl any new mount points or stop crawling
+	 * any mount points. We do it this way instead of listening
+	 * for the same HAL signals in the processor because the
+	 * processor checks state and at the time, we are PAUSED which
+	 * causes us state machine problems. 
+	 *
+	 * This is the easiest way to do it.
+	 */
+	if (!mpu->no_crawling) {
+		if (mpu->was_added) {
+			tracker_processor_mount_point_added (private->processor,
+							     mpu->udi,
+							     mpu->mount_point);
+		} else {
+			tracker_processor_mount_point_removed (private->processor,
+							       mpu->udi,
+							       mpu->mount_point);
+		}
+	}
+
+	mount_point_update_free (mpu);
+}
+
+static void
+mount_point_added_cb (TrackerHal  *hal,
+		      const gchar *udi,
+		      const gchar *mount_point,
+		      gpointer	   user_data)
+{
+	TrackerMainPrivate *private;
+	MountPointUpdate   *mpu;
+	
+	private = g_static_private_get (&private_key);
+
+	private->mount_points_to_set++;
+
+	tracker_status_set_is_paused_for_dbus (TRUE);
+
+	g_message ("Indexer is being notified about added volume with UDI:");
+	g_message ("  %s", udi);
+
+	mpu = mount_point_update_new (udi, mount_point, FALSE, TRUE);
+	org_freedesktop_Tracker_Indexer_volume_update_state_async (tracker_dbus_indexer_get_proxy (), 
+								   udi,
+								   mount_point,
+								   TRUE,
+								   mount_point_set_cb,
+								   mpu);
+}
+
+static void
+mount_point_removed_cb (TrackerHal  *hal,
+			const gchar *udi,
+			const gchar *mount_point,
+			gpointer     user_data)
+{
+	TrackerMainPrivate *private;
+	MountPointUpdate   *mpu;
+
+	private = g_static_private_get (&private_key);
+
+	private->mount_points_to_set++;
+
+	tracker_status_set_is_paused_for_dbus (TRUE);
+
+	g_message ("Indexer is being notified about removed volume with UDI:");
+	g_message ("  %s", udi);
+
+	mpu = mount_point_update_new (udi, mount_point, FALSE, FALSE);
+	org_freedesktop_Tracker_Indexer_volume_update_state_async (tracker_dbus_indexer_get_proxy (), 
+								   udi,
+								   mount_point,
+								   FALSE,
+								   mount_point_set_cb,
+								   mpu);
+}
 
 static void
 set_up_mount_points_cb (DBusGProxy *proxy, 
 			GError     *error,
 			gpointer    user_data)
 {
+	TrackerMainPrivate *private;
 	TrackerHal *hal;
 	GList *roots, *l;
 
 	if (error) {
-		g_critical ("Couldn't disable all volumes, %s", 
-			    error->message);
+		g_critical ("Indexer couldn't disable all volumes, %s", 
+			    error ? error->message : "no error given");
 		g_error_free (error);
+		tracker_shutdown ();
 		return;
 	}
 
-	g_message ("Indexer is being notified about ALL UDIs");
+	g_message ("  Done");
+
+	private = g_static_private_get (&private_key);
 
 	hal = user_data;
 	roots = tracker_hal_get_removable_device_udis (hal);
-	
-	for (l = roots; l; l = l->next) {
-		gchar       *udi;
-		const gchar *mount_point;
-		gboolean     is_mounted;
 
-		udi = l->data;
-		mount_point = tracker_hal_udi_get_mount_point (hal, udi);
-		is_mounted = tracker_hal_udi_get_is_mounted (hal, udi);
+	if (roots) {
+		g_message ("Indexer is being notified about volume states with UDIs:");
 
-		g_message ("  %s", udi);
+		for (l = roots; l; l = l->next) {
+			MountPointUpdate *mpu;
 
-		org_freedesktop_Tracker_Indexer_volume_update_state_async (tracker_dbus_indexer_get_proxy (), 
-									   udi,
-									   mount_point,
-									   is_mounted,
-									   mount_point_set_cb,
-									   g_strdup (udi));
+			private->mount_points_to_set++;
+			mpu = mount_point_update_new (l->data, 
+						      tracker_hal_udi_get_mount_point (hal, l->data),
+						      TRUE, 
+						      tracker_hal_udi_get_is_mounted (hal, l->data));	
+		
+			g_message ("  %s", mpu->udi);
+		
+			org_freedesktop_Tracker_Indexer_volume_update_state_async (tracker_dbus_indexer_get_proxy (), 
+										   mpu->udi,
+										   mpu->mount_point,
+										   mpu->was_added,
+										   mount_point_set_cb,
+										   mpu);
+		}
+
+		g_list_free (roots);
+	} else {
+		g_message ("Indexer does not need to be notified of volume states, none to set");
+
+		tracker_status_set_is_paused_for_dbus (FALSE);
+
+		/* Make sure we stop the main loop used to set up
+		 * mount points on start up if we have no removable
+		 * devices to update in the indexer.
+		 */
+		private->mount_points_up = TRUE;
+
+		/* We need to stop the main loop, we started
+		 * it so we could wait for mount points to be
+		 * set up successfully in main().
+		 */
+		g_main_loop_quit (private->main_loop);
 	}
-
-	g_list_free (roots);
 }
 
 static void
 set_up_mount_points (TrackerHal *hal)
 {
+	TrackerMainPrivate *private;
+
+	private = g_static_private_get (&private_key);
+
+	tracker_status_set_is_paused_for_dbus (TRUE);
+
+	private->mount_points_up = FALSE;
+
 	g_message ("Indexer is being notified to disable all volumes");
 	org_freedesktop_Tracker_Indexer_volume_disable_all_async (tracker_dbus_indexer_get_proxy (), 
 								  set_up_mount_points_cb,
 								  hal);
 }
 
-static void
-set_up_throttle (TrackerHal    *hal,
-		 TrackerConfig *config)
-{
-	gint throttle;
-
-	/* If on a laptop battery and the throttling is default (i.e.
-	 * 0), then set the throttle to be higher so we don't kill
-	 * the laptop battery.
-	 */
-	throttle = tracker_config_get_throttle (config);
-
-	if (tracker_hal_get_battery_in_use (hal)) {
-		g_message ("We are running on battery");
-
-		if (throttle == THROTTLE_DEFAULT) {
-			tracker_config_set_throttle (config,
-						     THROTTLE_DEFAULT_ON_BATTERY);
-			g_message ("Setting throttle from %d to %d",
-				   throttle,
-				   THROTTLE_DEFAULT_ON_BATTERY);
-		} else {
-			g_message ("Not setting throttle, it is currently set to %d",
-				   throttle);
-		}
-	} else {
-		g_message ("We are not running on battery");
-
-		if (throttle == THROTTLE_DEFAULT_ON_BATTERY) {
-			tracker_config_set_throttle (config,
-						     THROTTLE_DEFAULT);
-			g_message ("Setting throttle from %d to %d",
-				   throttle,
-				   THROTTLE_DEFAULT);
-		} else {
-			g_message ("Not setting throttle, it is currently set to %d",
-				   throttle);
-		}
-	}
-}
-
-static void
-notify_battery_in_use_cb (GObject    *gobject,
-			  GParamSpec *arg1,
-			  gpointer    user_data)
-{
-	set_up_throttle (TRACKER_HAL (gobject),
-			 TRACKER_CONFIG (user_data));
-}
-
 #endif /* HAVE_HAL */
 
 static gboolean
-start_cb (gpointer user_data)
+start_processor_cb (gpointer user_data)
 {
 	TrackerMainPrivate *private;
 
-	if (!tracker_status_get_is_ready ()) {
-		return FALSE;
-	}
-
 	private = g_static_private_get (&private_key);
 
 	if (private->processor) {
@@ -863,14 +944,12 @@ gint
 main (gint argc, gchar *argv[])
 {
 	GOptionContext		   *context = NULL;
-	GOptionGroup		   *group;
 	GError			   *error = NULL;
 	TrackerMainPrivate	   *private;
 	TrackerConfig		   *config;
 	TrackerLanguage		   *language;
 	TrackerHal		   *hal;
 	TrackerDBIndex		   *file_index;
-	TrackerDBIndex		   *file_update_index;
 	TrackerDBIndex		   *email_index;
 	TrackerRunningLevel	    runtime_level;
 	TrackerDBManagerFlags	    flags = 0;
@@ -903,25 +982,7 @@ main (gint argc, gchar *argv[])
 	 * usage string - Usage: COMMAND <THIS_MESSAGE>
 	 */
 	context = g_option_context_new (_("- start the tracker daemon"));
-
-	/* Daemon group */
-	group = g_option_group_new ("daemon",
-				    _("Daemon Options"),
-				    _("Show daemon options"),
-				    NULL,
-				    NULL);
-	g_option_group_add_entries (group, entries_daemon);
-	g_option_context_add_group (context, group);
-
-	/* Indexer group */
-	group = g_option_group_new ("indexer",
-				    _("Indexer Options"),
-				    _("Show indexer options"),
-				    NULL,
-				    NULL);
-	g_option_group_add_entries (group, entries_indexer);
-	g_option_context_add_group (context, group);
-
+	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_parse (context, &argc, &argv, &error);
 	g_option_context_free (context);
 
@@ -931,8 +992,12 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
-	/* Print information */
-	g_print ("\n" ABOUT "\n" LICENSE "\n");
+	if (version) {
+		/* Print information */
+		g_print ("\n" ABOUT "\n" LICENSE "\n");
+		return EXIT_SUCCESS;
+	}
+
 	g_print ("Initializing trackerd...\n");
 
 	initialize_signal_handler ();
@@ -942,21 +1007,8 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
-	/* Set IO priority */
-	tracker_ioprio_init ();
-
-	/* nice() uses attribute "warn_unused_result" and so complains
-	 * if we do not check its returned value. But it seems that
-	 * since glibc 2.2.4, nice() can return -1 on a successful
-	 * call so we have to check value of errno too. Stupid...
-	 */
-	if (nice (19) == -1 && errno) {
-		const gchar *str;
-
-		str = g_strerror (errno);
-		g_message ("Couldn't set nice value to 19, %s",
-			   str ? str : "no error given");
-	}
+	/* This makes sure we don't steal all the system's resources */
+	initialize_priority ();
 
 	/* This makes sure we have all the locations like the data
 	 * dir, user data dir, etc all configured.
@@ -1011,6 +1063,10 @@ main (gint argc, gchar *argv[])
 
 	initialize_directories ();
 
+	if (!tracker_dbus_init (config)) {
+		return EXIT_FAILURE;
+	}
+
 	/* Initialize other subsystems */
 	tracker_log_init (private->log_filename, tracker_config_get_verbosity (config));
 	g_print ("Starting log:\n  File:'%s'\n", private->log_filename);
@@ -1019,16 +1075,23 @@ main (gint argc, gchar *argv[])
 
 	tracker_nfs_lock_init (tracker_config_get_nfs_locking (config));
 
-	if (!tracker_dbus_init (config)) {
-		return EXIT_FAILURE;
-	}
+#ifdef HAVE_HAL
+	hal = tracker_hal_new ();
 
-	tracker_status_init (config);
+	g_signal_connect (hal, "mount-point-added",
+			  G_CALLBACK (mount_point_added_cb),
+			  NULL);
+	g_signal_connect (hal, "mount-point-removed",
+			  G_CALLBACK (mount_point_removed_cb),
+			  NULL);
+#endif /* HAVE_HAL */
+
+	tracker_status_init (config, hal);
 
 	tracker_module_config_init ();
 
 	tracker_turtle_init ();
-	tracker_thumbnailer_init (config, 30);
+	tracker_thumbnailer_init (config);
 
 	flags |= TRACKER_DB_MANAGER_REMOVE_CACHE;
 	index_flags |= TRACKER_DB_INDEX_MANAGER_READONLY;
@@ -1053,20 +1116,6 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
-#ifdef HAVE_HAL
-	hal = tracker_hal_new ();
-
-	g_signal_connect (hal, "notify::battery-in-use",
-			  G_CALLBACK (notify_battery_in_use_cb),
-			  config);
-	g_signal_connect (hal, "mount-point-added",
-			  G_CALLBACK (mount_point_added_cb),
-			  NULL);
-	g_signal_connect (hal, "mount-point-removed",
-			  G_CALLBACK (mount_point_removed_cb),
-			  NULL);
-#endif /* HAVE_HAL */
-
 	/*
 	 * Check instances running
 	 */
@@ -1087,24 +1136,24 @@ main (gint argc, gchar *argv[])
 		break;
 	}
 
+	/*
+	 * Set up databases
+	 */
 	if (!initialize_databases ()) {
 		return EXIT_FAILURE;
 	}
 
 	file_index = tracker_db_index_manager_get_index (TRACKER_DB_INDEX_FILE);
-	file_update_index = tracker_db_index_manager_get_index (TRACKER_DB_INDEX_FILE_UPDATE);
 	email_index = tracker_db_index_manager_get_index (TRACKER_DB_INDEX_EMAIL);
 
 	if (!TRACKER_IS_DB_INDEX (file_index) ||
-	    !TRACKER_IS_DB_INDEX (file_update_index) ||
 	    !TRACKER_IS_DB_INDEX (email_index)) {
-		g_critical ("Could not create indexer for all indexes (file, file-update, email)");
+		g_critical ("Could not create indexer for all indexes (file, email)");
 		return EXIT_FAILURE;
 	}
 
 	tracker_data_manager_init (config, language, file_index, email_index);
-	tracker_xesam_manager_init ();
-	tracker_cleanup_init ();
+	tracker_volume_cleanup_init ();
 
 #ifdef HAVE_HAL
 	/* We set up the throttle and mount points here. For the mount
@@ -1112,11 +1161,31 @@ main (gint argc, gchar *argv[])
 	 * we have to have already initialised the databases if we
 	 * are going to do that.
 	 */
-	set_up_throttle (hal, config);
 	set_up_mount_points (hal);
+	
+	/* Wait until we have these set up. */
+	private->main_loop = g_main_loop_new (NULL, FALSE);
+	g_main_loop_run (private->main_loop);
+	g_main_loop_unref (private->main_loop);
+	private->main_loop = NULL;
 #endif /* HAVE_HAL */
 
+	if (private->shutdown) {
+		goto shutdown;
+	}
+
+	/* 
+	 * Start public interfaces (DBus, push modules, etc)
+	 */
 	private->processor = tracker_processor_new (config, hal);
+		
+	if (force_reindex &&
+	    g_file_test (get_ttl_backup_filename (), G_FILE_TEST_EXISTS)) {
+		g_debug ("Setting callback for crawling finish detection");
+		g_signal_connect (private->processor, "finished", 
+				  G_CALLBACK (crawling_finished_cb), 
+				  NULL);
+	}
 
 	/* Make Tracker available for introspection */
 	if (!tracker_dbus_register_objects (config,
@@ -1126,29 +1195,30 @@ main (gint argc, gchar *argv[])
 					    private->processor)) {
 		return EXIT_FAILURE;
 	}
+	
+	tracker_push_init (config);
+	
+	private->public_interfaces_up = TRUE;
 
-#ifdef HAVE_EVOLUTION_PLUGIN
-	tracker_evolution_init (config);
-#endif
+	/* Signal to clients current statistics */
+	tracker_daemon_signal_statistics ();
 
 	g_message ("Waiting for DBus requests...");
 
-	/* Set our status as running, if this is FALSE, threads stop
-	 * doing what they do and shutdown.
+	/* 
+	 * Start processor
 	 */
-	tracker_status_set_is_ready (TRUE);
-
 	if (!tracker_status_get_is_readonly ()) {
 		gint seconds;
-
+		
 		seconds = tracker_config_get_initial_sleep (config);
-
+		
 		if (seconds > 0) {
-			g_message ("Waiting %d seconds before starting",
+			g_message ("Waiting %d seconds before starting processor",
 				   seconds);
-			g_timeout_add (seconds * 1000, start_cb, NULL);
+			g_timeout_add_seconds (seconds, start_processor_cb, NULL);
 		} else {
-			g_idle_add (start_cb, NULL);
+			g_idle_add (start_processor_cb, NULL);
 		}
 	} else {
 		/* We set the state here because it is not set in the
@@ -1158,18 +1228,17 @@ main (gint argc, gchar *argv[])
 		tracker_status_set_and_signal (TRACKER_STATUS_IDLE);
 	}
 
-	if (flags & TRACKER_DB_MANAGER_FORCE_REINDEX ||
-	    g_file_test (get_ttl_backup_filename (), G_FILE_TEST_EXISTS)) {
-		backup_restore_on_crawling_finished (private->processor);
-	}
-
-	if (tracker_status_get_is_ready ()) {
+	/* 
+	 * Start main loop
+	 */
+	if (!private->shutdown) {
 		private->main_loop = g_main_loop_new (NULL, FALSE);
 		g_main_loop_run (private->main_loop);
 	}
 
+shutdown:
 	/*
-	 * Shutdown the daemon
+	 * Shutdown main loop
 	 */
 	g_message ("Shutdown started");
 
@@ -1177,15 +1246,6 @@ main (gint argc, gchar *argv[])
 
 	g_timeout_add_full (G_PRIORITY_LOW, 5000, shutdown_timeout_cb, NULL, NULL);
 
-	g_message ("Waiting for indexer to finish");
-	org_freedesktop_Tracker_Indexer_shutdown (tracker_dbus_indexer_get_proxy (), &error);
-
-	if (error) {
-		g_message ("Could not shutdown the indexer, %s", error->message);
-		g_message ("Continuing anyway...");
-		g_error_free (error);
-	}
-
 	g_message ("Cleaning up");
 	if (private->processor) {
 		/* We do this instead of let the private data free
@@ -1196,18 +1256,15 @@ main (gint argc, gchar *argv[])
 		private->processor = NULL;
 	}
 
-	shutdown_indexer ();
 	shutdown_databases ();
 	shutdown_directories ();
 
 	/* Shutdown major subsystems */
-	
-#ifdef HAVE_EVOLUTION_PLUGIN
-	tracker_evolution_shutdown ();
-#endif
+	if (private->public_interfaces_up) {
+		tracker_push_shutdown ();
+	}
 
-	tracker_cleanup_shutdown ();
-	tracker_xesam_manager_shutdown ();
+	tracker_volume_cleanup_shutdown ();
 	tracker_dbus_shutdown ();
 	tracker_db_manager_shutdown ();
 	tracker_db_index_manager_shutdown ();
@@ -1220,9 +1277,6 @@ main (gint argc, gchar *argv[])
 	tracker_log_shutdown ();
 
 #ifdef HAVE_HAL
-	g_signal_handlers_disconnect_by_func (hal,
-					      notify_battery_in_use_cb,
-					      config);
 	g_signal_handlers_disconnect_by_func (hal,
 					      mount_point_added_cb,
 					      NULL);
@@ -1238,6 +1292,8 @@ main (gint argc, gchar *argv[])
 
 	shutdown_locations ();
 
+	g_static_private_free (&private_key);
+
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
@@ -1250,11 +1306,27 @@ tracker_shutdown (void)
 
 	private = g_static_private_get (&private_key);
 
-	tracker_status_set_is_ready (FALSE);
+	if (private) {
+		if (private->processor) {
+			tracker_processor_stop (private->processor);
+		}
 
-	tracker_processor_stop (private->processor);
+		if (private->main_loop) {
+			g_main_loop_quit (private->main_loop);
+		}
 
-	g_main_loop_quit (private->main_loop);
+		private->shutdown = TRUE;
+	}
+}
+
+const gchar *
+tracker_get_data_dir (void)
+{
+	TrackerMainPrivate *private;
+
+	private = g_static_private_get (&private_key);
+
+	return private->data_dir;
 }
 
 const gchar *
